/***************************************************************************
    smb4k.h  -  The main class of Smb4K.
                             -------------------
    begin                : Sam Mr  1 14:57:21 CET 2003
    copyright            : (C) 2003 by Alexander Reinholdt
    email                : dustpuppy@mail.berlios.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SMB4K_H
#define SMB4K_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

// Qt forward declarations
#include <qlabel.h>

// include files for KDE
#include <kuniqueapplication.h>
#include <kdockwidget.h>
#include <kaccel.h>
#include <kaction.h>
#include <klistview.h>
#include <kiconview.h>
#include <kprogress.h>

// application specific includes
#include "core/smb4kcore.h"
#include "smb4ksystemtray.h"
#include "widgets/smb4kbookmarkmenuwidget.h"
#include "widgets/smb4ksharewidget.h"
#include "widgets/smb4kbrowserwidget.h"
#include "widgets/smb4ksearchdialog.h"

/**
  * The base class for Smb4K application windows. It sets up the main
  * window and reads the config file as well as providing a menubar, toolbar
  * and statusbar. An instance of Smb4KView creates your center view, which is connected
  * to the window's Doc object.
  * Smb4KApp reimplements the methods that KMainWindow provides for main window handling and supports
  * full session management as well as using KActions.
  * @see KMainWindow
  * @see KApplication
  * @see KConfig
  *
  * @author Source Framework Automatically Generated by KDevelop, (c) The KDevelop Team.
  * @version KDevelop version 1.2 code generation
  */
class Smb4KApp : public KDockMainWindow
{
  Q_OBJECT

  friend class Smb4KBrowserWidget;
  friend class Smb4KShareWidget;
  friend class Smb4KSystemTray;
  friend class Smb4KBookmarkMenuWidget;

  public:
    /**
     * The constructor
     */
    Smb4KApp( QWidget* parent=0, const char *name=0 );

    /**
     * The destructor
     */
    ~Smb4KApp();

    /**
     * Returns the pointer to the network view.
     */
    Smb4KBrowserWidget *networkView() { return m_network_view; }

    /**
     * Returns the pointer to the search tab.
     */
    Smb4KSearchDialog *searchDialog() { return m_search_dialog; }

    /**
     * read general Options again and initialize all variables like the recent file list
     */
    void readOptions();


  protected:
    /** save general Options like all bar positions and status as well as the geometry and the recent file list to the configuration
     * file
     */
    void saveOptions();

    /**
     * Initializes the KActions of the application
     */
    void initActions();

    /**
     * Sets up the statusbar for the main window by initialzing a statuslabel.
     */
    void initStatusBar();

    /**
     * Creates the centerwidget of the KTMainWindow instance and sets it as the view
     */
    void initView();

    /**
     * Initializes the core applications.
     */
    void initCore();

    /**
     * Plugs several items into the menu bar that could not be loaded otherwise.
     */
    void setupMenuBar();

    /**
     * Sets up the system tray icon.
     */
    void setupSystemTray();

    /**
     * Reimplemented from KMainWindow.
     */
    bool queryExit();

    /**
     * Reimplemented from KMainWindow.
     */
    bool queryClose();

  public slots:
    /**
     * This slot quits the application.
     */
    void slotQuit();

    /**
     * With this slot you are able to set a status message in the
     * status bar.
     */
    void slotStatusMsg(const QString &text);

    /**
     * Opens the configuration dialog.
     */
    void slotSmb4KOptionsDlg();

  protected slots:
    /**
     * Aborts any action of the program.
     */
    void slotAbort();

    /**
     * Sets the progress bar.
     */
    void slotSetProgress();

    /**
     * This slot is invoked, if the options have to be reread.
     */
    void slotReadOptions();

    /**
     * This slot is activated when the run state of the core changed. It handles
     * the messages of the status bar and the enabling and disabling of the
     * Abort and other actions.
     */
    void slotCoreRunStateChanged();

    /**
     * This slot shows the search dialog if the appropriate shortcut is
     * pressed. The keyboard focus will be set to the combo box.
     */
    void slotShowSearchDialog();

    /**
     * This slot shows the network browser if the appropriate shortcut is
     * pressed. The keyboard focus will be set to the list view.
     */
    void slotShowNetworkBrowser();

    /**
     * This slot will give the keyboard focus to the shares view if the
     * appropriate shortcut is pressed.
     */
    void slotJumpToSharesView();

    /**
     * This slot is invoked when the user changes the icon size, etc. It is
     * connected to the signal KApplication::iconChanged().
     *
     * @param group       The icon group
     */
    void slotIconChanged( int group );

  private:
    /**
     * Determines whether Smb4K should only start docked or if the main window
     * should be shown.
     */
    bool m_startDocked;
    /**
     * The docking class
     */
    Smb4KSystemTray *sys_tray;
    /**
     * Label to hold the WINS server info in the status bar.
     */
    QLabel *m_winsLabel;
    /**
     * Label to hold the browse list retrieval info in the status bar.
     */
    QLabel *m_listLabel;
    /**
     * The progress bar for the status bar.
     */
    KProgress *m_progress;
    /**
     * The timer, that drives the progress bar.
     */
    QTimer *m_progTimer;
    /**
     * The share view.
     */
    Smb4KShareWidget *m_share_view;
    /**
     * The network view.
     */
    Smb4KBrowserWidget *m_network_view;
    /**
     * The search tab.
     */
    Smb4KSearchDialog *m_search_dialog;
};

#endif // SMB4K_H
