/***************************************************************************
    smb4ksystemtray.h  - The widget that handles the docking to the
             system tray.
                             -------------------
    begin                : Mon Jan 12 2004
    copyright            : (C) 2004 by Franck Babin
                           (C) 2004 by Alexander Reinholdt
    email                : babinfranck@yahoo.ca
                           dustpuppy@mail.berlios.de
 ***************************************************************************/

/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful, but   *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
 *   MA  02111-1307 USA                                                    *
 ***************************************************************************/

#ifndef SMB4KSYSTEMTRAY_H
#define SMB4KSYSTEMTRAY_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

// Qt includes
#include <qwidget.h>

// KDE includes
#include <ksystemtray.h>
#include <kaction.h>


class Smb4KSystemTray : public KSystemTray
{
  Q_OBJECT

  public:
    /**
     * The constructor.
     */
    Smb4KSystemTray( bool dock_now, QWidget *parent = 0, const char *name = 0 );
    /**
     * The destructor.
     */
    ~Smb4KSystemTray();
    /**
     * Returns TRUE if the main application has been docked
     * to the system tray and FALSE otherwise.
     */
    bool isDocked() { return m_docked; }
    /**
     * Inserts an action to the given position.
     *
     * @param action    The action that is to be plugged into the system tray.
     *
     * @param pos       The position where the action should be plugged.
     */
    void insertAction( KAction *action, int pos );

  public slots:
    /**
     * Docks the main widget to the system tray.
     */
    void dock();
    /**
     * Undocks the window from the system tray.
     */
    void undock();

  private:
    /**
     * Sets up the context menu.
     */
    void setupContextMenu();
    /**
     * When the application is currently docked, this is set to
     * TRUE otherwise it is false.
     */
    bool m_docked;
};

#endif
